# -*- encoding: utf-8 -*-
"""AMQP Specifications and Classes"""
__author__ = 'Gavin M. Roy'
__email__ = 'gavinmroy@gmail.com'
__since__ = '2011-09-23'
__version__ = version = '3.2.1'

__all__ = [
    'body', 'decode', 'commands', 'constants', 'encode', 'exceptions', 'frame',
    'header', 'heartbeat'
]
